import { IPosition } from "./Interface.type";

export const calculateOptimalPosition = (
  targetId: string,
  popupWidth: number = 50,
  popupHeight: number = 50,
  offset?: { x?: number; y?: number }
): IPosition | null => {
  const targetElement = document.getElementById(targetId);
  if (!targetElement) {
    console.warn(`Element with ID '${targetId}' not found.`);
    return null;
  }

  const targetRect = targetElement.getBoundingClientRect();
  const viewportWidth = window.innerWidth;
  const viewportHeight = window.innerHeight;
  const scrollX = window.scrollX;
  const scrollY = window.scrollY;

  // Set default offset values
  const defaultOffset =   5;
  const offsetX = (offset?.x ?? defaultOffset) + 100; // Extra spacing
  const offsetY = (offset?.y ?? defaultOffset) + 50;

  // Determine if the screen size is mobile/tablet
  const isMobile = viewportWidth <= 600;

  let position: IPosition;

  if (isMobile) {
    // Mobile and tablet view logic
    position = {
      top: Math.max(targetRect.top + scrollY + offsetY, 10),
      left: Math.max(targetRect.left + scrollX, 10),
      transformOrigin: "top left",
    };
  } else {
    // Desktop view logic
    const isRightSide = targetRect.left > viewportWidth / 2;

    let topPosition: number;
    if (targetRect.bottom + popupHeight + offsetY <= viewportHeight + scrollY) {
      topPosition = targetRect.bottom + scrollY + offsetY; // Position below
    } else if (targetRect.top - popupHeight - offsetY >= scrollY) {
      topPosition = targetRect.top + scrollY - popupHeight - offsetY; // Position above
    } else {
      topPosition = Math.max(
        scrollY + defaultOffset,
        Math.min(
          scrollY + (viewportHeight - popupHeight) / 2,
          scrollY + viewportHeight - popupHeight - defaultOffset
        )
      ); // Center vertically
    }

    let leftPosition: number;
    if (isRightSide) {
      if (targetRect.left - popupWidth - offsetX >= scrollX) {
        leftPosition = targetRect.left + scrollX - popupWidth - offsetX; // Position left
      } else {
        leftPosition = Math.max(
          scrollX + defaultOffset,
          Math.min(
            scrollX + (viewportWidth - popupWidth) / 2,
            scrollX + viewportWidth - popupWidth - defaultOffset
          )
        ); // Center horizontally
      }
    } else {
      if (targetRect.right + popupWidth + offsetX <= scrollX + viewportWidth) {
        leftPosition = targetRect.right + scrollX + offsetX; // Position right
      } else {
        leftPosition = Math.max(
          scrollX + defaultOffset,
          Math.min(
            scrollX + (viewportWidth - popupWidth) / 2,
            scrollX + viewportWidth - popupWidth - defaultOffset
          )
        ); // Center horizontally
      }
    }

    let transformOrigin: string;
    if (topPosition < targetRect.top + scrollY) {
      transformOrigin = "bottom";
    } else if (topPosition > targetRect.bottom + scrollY) {
      transformOrigin = "top";
    } else {
      transformOrigin = "center";
    }

    if (leftPosition < targetRect.left + scrollX) {
      transformOrigin += " right";
    } else if (leftPosition > targetRect.right + scrollX) {
      transformOrigin += " left";
    } else {
      transformOrigin += " center";
    }

    position = {
      top: topPosition,
      left: leftPosition,
      transformOrigin,
    };
  }

  return position;
};
