import { useEffect, useState } from "react";
import Overlay from "../Overlay/Overlay";
import { calculateOptimalPosition } from "../OptimalPosition/CalculateOptimalPosition";
import "../Overlay/Overlay.css";
import "./Tourguide.css"; // Custom CSS file
import { useViewport } from "../../Hooks/useViewport";
import { ITourGuideProps, ColorConfig } from "./Interface.type";
import { IPosition } from "../OptimalPosition/Interface.type";

export const TourGuide: React.FC<ITourGuideProps> = ({
  steps,
  colorConfig: userColorConfig = {},
}) => {
  const viewportWidth = useViewport();
  const isMobile = viewportWidth <= 600;
  const isTablet = viewportWidth > 600 && viewportWidth <= 1024;

  const [isVisible, setIsVisible] = useState(false);
  const [currentStep, setCurrentStep] = useState(0);
  const [popupPosition, setPopupPosition] = useState<IPosition | null>(null);

  const [colorConfig] = useState<ColorConfig>({
    Modal: userColorConfig.Modal || "",
    textColor: userColorConfig.textColor || "",
    buttonColor: userColorConfig.buttonColor || "",
    overlayBackgroundColor: userColorConfig.overlayBackgroundColor || "",
    highlightColor: userColorConfig.highlightColor || "",
  });

  useEffect(() => {
    if (steps.length) setIsVisible(true);
  }, [steps]);

  const updatePopupPosition = () => {
    const step = steps[currentStep];
    const position = calculateOptimalPosition(step.id, 200, 100, {
      x: 10,
      y: 10,
    });
    setPopupPosition(position);
  };

  useEffect(() => {
    updatePopupPosition();

    const element = document.getElementById(steps[currentStep]?.id);
    if (element) {
      element.scrollIntoView({ behavior: "smooth", block: "center" });
    }

    const handleScroll = () => {
      updatePopupPosition();
    };

    window.addEventListener("scroll", handleScroll);
    return () => {
      window.removeEventListener("scroll", handleScroll);
    };
  }, [currentStep, steps]);

  useEffect(() => {
    const handleResize = () => {
      updatePopupPosition();
    };

    window.addEventListener("resize", handleResize);
    return () => window.removeEventListener("resize", handleResize);
  }, [currentStep, steps]);

  // Function to handle text-to-speech
  const handleSpeech = () => {
    const step = steps[currentStep];
    const speech = new SpeechSynthesisUtterance();
    speech.text = `${step.title}. ${step.description}`;
    speech.lang = "en-US"; // You can change this language as needed
    window.speechSynthesis.speak(speech);
  };

  // Keyboard Navigation
  useEffect(() => {
    const handleKeyDown = (event: KeyboardEvent) => {
      if (!isVisible) return;

      if (event.key === "ArrowRight") {
        handleNext();
      } else if (event.key === "ArrowLeft") {
        handlePrevious();
      } else if (event.key === "Escape") {
        handleClose();
      }
    };

    window.addEventListener("keydown", handleKeyDown);
    return () => {
      window.removeEventListener("keydown", handleKeyDown);
    };
  }, [isVisible, currentStep, steps]);

  const handleNext = () => setCurrentStep((prev) => Math.min(prev + 1, steps.length - 1));
  const handlePrevious = () => setCurrentStep((prev) => Math.max(prev - 1, 0));
  const handleClose = () => setIsVisible(false);
  const handleSkip = () => setIsVisible(false);
  const handleRestart = () => {
    setCurrentStep(0);
    setIsVisible(true);
  };

  const {
    overlayBackgroundColor,
    highlightColor,
  } = colorConfig;

  return (
    <>
      <button className="tour-restart-button" onClick={handleRestart}>
        Restart Tour
      </button>

      {isVisible && popupPosition && (
        <>
          <Overlay
            activeElementId={steps[currentStep]?.id}
            onClose={handleClose}
            colorConfig={{
              overlayBackgroundColor: overlayBackgroundColor || "",
              highlightColor: highlightColor || "",
            }}
          />

          <div
            className={`tour-overlay-modal ${isMobile ? "mobile" : ""} ${isTablet ? "tablet" : ""}`}
            style={{
              top: popupPosition?.top || "auto",
              left: popupPosition?.left || "auto",
              transformOrigin: popupPosition?.transformOrigin,
            }}
          >
            <button className="tour-close-button" onClick={handleClose}>&times;</button>

            <h3 className="tour-title">{steps[currentStep]?.title}</h3>
            <p className="tour-description">{steps[currentStep]?.description}</p>

            {steps[currentStep]?.logoUrl && (
              <div className="tour-logo-container">
                <img
                  src={steps[currentStep]?.logoUrl}
                  alt="Logo"
                  className="tour-logo"
                />
              </div>
            )}

            <div className="tour-button-container">
              <button className="tour-button" onClick={handleSkip}>
                Skip
              </button>
              <button
                className="tour-button"
                onClick={handlePrevious}
                disabled={currentStep === 0}
              >
                Previous
              </button>
              <span className="tour-step-indicator">
                {currentStep + 1} / {steps.length}
              </span>
              <button
                className="tour-button"
                onClick={handleNext}
                disabled={currentStep === steps.length - 1}
              >
                Next
              </button>
              <button className="mic-button" onClick={handleSpeech}>
                <i className="fa fa-microphone"></i>
              </button>
            </div>
          </div>
        </>
      )}
    </>
  );
};
